use strict;
package IMPL::Web::TT::Form;

use base qw(IMPL::Web::TT::Control);

use IMPL::Class::Property;
use IMPL::DOM::Navigator::SchemaNavigator;
use IMPL::DOM::Property qw(_dom);

__PACKAGE__->PassThroughArgs;

BEGIN {
	public property base => prop_all;
	public property schema => prop_all;
	public property errors => prop_all;
	public property data => prop_all;
}

sub CTOR {
	my ($this) = @_;
	
	$this->base($this->nodeName) unless $this->base;
	
	die new IMPL::InvalidArgumentException('A schema is required for a form',$this->nodeName)
		unless eval { $this->schema->isa( typeof IMPL::DOM::Schema ) };
	
	die new IMPL::InvalidOperationException('Can\'t find a form definition in a schema',$this->nodeName,$this->base)
		unless $this->schema->selectNodes(sub { $_->nodeName eq 'ComplexNode' and $_->name eq $this->base });
		
	$this->errors([]) unless $this->errors;
}

sub makeControlArgs{
	my ($this,$path) = @_;
	
	my $navi = new IMPL::DOM::Navigator::SchemaNavigator($this->schema);
	my @path = ($this->base, split /\./,$path); 
	
	$navi->NavigateName($_) or die new IMPL::InvalidArgumentException(
		"Can't find a definition for an element",
		$_,
		$path,
		$this->element,
	) foreach @path;
	
	my $schema = $navi->Current;
	my $node = $this->data->selectSingleNode(@path);
	
	my @errors;
	
	if ($node) {
		@errors = grep { ( $_->Node || $_->Parent) == $node } @{$this->errors}; 
	}
	
	return {
		schema => $schema,
		sourceSchema => $navi->SourceSchemaNode,
		errors => \@errors,
		data => $node,
		formParameter => join '/', @path
	};
}

sub formErrors {
	my ($this) = @_;
	
	if (my $node = $this->data->selectSingleNode($this->base) ) {
		return [
			grep {
				( $_->Node || $_->Parent) == $node
			} @{$this->errors}
		];
	} else {
		return [];
	}
}


1;

__END__

=pod

=head1 NAME

C<IMPL::Web::TT::Form> -   

=head1 DESCRIPTION

  ,     
 

=cut